package PureMessage::Monitor::Analyzer::Threshold;

use strict;
use warnings;
use Carp;
use PureMessage::Monitor::Event;
use PureMessage::Monitor::TimedCounter;
use base qw( PureMessage::Monitor::EventHandler );

my $FIRED_KEY = 'fired';

my %DEFAULTS = (
    'threshold' => 5,
    'expiry'    => 300,
    );

# creates a new "threshold" analyzer, and sets up the events/data points that
# we're analyzing.  Acceptable options:
#   expiry      - expiry period for data being monitored (default 300s)
#   threshold   - threshold to be reached before signalling action event
#                 (default 5)
#   field       - field in Event data to watch if/when it exceeds threshold
#                 (REQUIRED)
sub new {
    my $proto = shift;

    # create a new object
    my $self = $proto->SUPER::new( @_ );

    # sanity check parameters
    if (!$self->{'field'}) {
        croak "can't create threshold analyzer without field to monitor";
    }

    # use defaults where necessary
    foreach my $key (keys %DEFAULTS) {
        if (!defined $self->{$key}) {
            $self->{$key} = $DEFAULTS{$key};
        }
    }

    # create a timed counter to monitor our data field
    my $name  = $self->name();
    my $field = $self->{'field'};
    $self->{'counter'} = PureMessage::Monitor::TimedCounter->new(
                            "$name/$field",
                            'expiry'    => $self->{'expiry'},
                            );

    # return the newly created object back to the caller.
    return $self;
}

# runs the analyzer, analyzing a single event
sub analyze_event {
    my ($self, $queue, $event) = @_;

    # Get the field being monitored out of the Event.
    my $name  = $self->name();
    my $field = $self->{'field'};
    my $value = $event->param( $field );
    if (defined $value) {
        # increment the count for our data field being monitored
        my $counter = $self->{'counter'};
        my $timestamp = $event->param( 'time_stamp' );
        $counter->increment( $value, 'now' => $timestamp );

        # If we've exceeded the threshold, fire an action event off into the
        # queue.
        my $count = $counter->count( $value );
        my $threshold = $self->{'threshold'};
        $self->log( 'DEBUG', "analysis '$name' $field:$value has count $count, threshold $threshold" );
        if ($count >= $threshold) {
            if (!$counter->flag( $value, $FIRED_KEY )) {
                my %event_data = (
                                  # meta-data
                                  'name'          => $name,
                                  'time_stamp'    => time(),
                                  # analysis data point(s)
                                  'field'         => $field,          # field being monitored
                                  'value'         => $value,          # value which hit threshold
                                  'threshold'     => $threshold,      # threshold/maximum
                                  'count'         => $count,          # current count
                                  );
                $self->log( 'NOTICE', "analysis '$name' threshold of $threshold reached for $field:$value ($count), firing event" );
                my $new_event = PureMessage::Monitor::Event->new( %event_data );
                $queue->add_item( $new_event );
                $counter->flag( $value, $FIRED_KEY, 1 );
            }
        }
        else {
            $counter->flag( $value, $FIRED_KEY, 0 );
        }
    }
    $self->cleanup();
}

# cleans up our counters.  Only the analyzer truly knows what's being counted,
# so its responsible for providing a method to periodically tell our counters
# to go clean themselves up and purge any old/stale data.
sub cleanup {
    my ($self) = @_;
    $self->log( 'DEBUG', 'cleaning up counters and purging old/stale data' );
    $self->{'counter'}->purge();
}

1;

=head1 NAME

PureMessage::Monitor::Analyzer::Threshold - Threshold analyzer

=head1 SYNOPSIS

    <analyzer too_much_spam_from_host>
        use = PureMessage::Monitor::analyzer::Threshold
        event = spam
        expiry = 5m
        threshold = 2
        field = relay_host
    </analyzer>

    <analyzer too_much_spam_from_addr>
        use = PureMessage::Monitor::analyzer::Threshold
        event = spam
        threshold = 2
        field = env_from
    </analyzer>

=cut
