package PureMessage::Monitor::Event;

use strict;
use warnings;
use Carp;
use POSIX qw(strftime);
use ActiveState::Config qw(:path);

# creates a new named event object
sub new {
    my ($proto, %data) = @_;

    # sanity check parameters
    if (!$data{'name'}) {
        croak "can't create un-named event";
    }
    if (!$data{'time_stamp'}) {
        croak "all events must be time stamped";
    }

    # create a new object and bless it into our namespace.
    my $class = ref($proto) || $proto;
    my $self = {
        %data,
        };
    bless $self, $class;

    # return the newly created object back to the caller.
    return $self;
}

# returns the event name
sub name {
    return shift->{'name'};
}

# turns the event into a hash and returns it back to the caller.
sub as_hash {
    my $self = shift;

    # start with all of our parameters
    my %hash = %{$self};

    # add in some meta/computed parameters
    $hash{'time_stamp_str'} = strftime( '%Y-%m-%dT%H:%M:%S', localtime($hash{'time_stamp'}) );

    # return the hash back to the caller
    return %hash;
}

# sets/queries an event parameter
sub param {
    my $self = shift;
    my $parm = shift;
    if (@_) {
        $self->{$parm} = $_[0];
    }
    return $self->{$parm}
}

# INTERNAL METHOD. create an AS::Config object that represents the event
sub _as_cfg {
    my $self = shift;

    my $cfg  = ActiveState::Config->new();
    my $root = $cfg->root();
    foreach my $key (sort keys %{$self}) {  # sort, so testing is easier
        my $val = $self->{$key};
        if (ref($val)) {
            path_values( $root, $key, $val );
        }
        else {
            path_value( $root, $key, $val );
        }
    }
    return $cfg;
}

# serializes the event into a string
sub as_string {
    my $self = shift;
    return $self->_as_cfg()->as_string();
}

# serializes the event into a string and saves to a file
sub save {
    my ($self, $filename) = @_;
    return $self->_as_cfg()->saveto( $filename );
}

# un-serializes, from a string
sub parse {
    my ($class, $string) = @_;

    # use AS::Config to help with serialization
    my $cfg  = ActiveState::Config->new();
    $cfg->parse( $string );
    my %hash = $cfg->as_hash();

    # bless the hash into an Event and return it to the caller
    my $event = \%hash;
    bless $event, $class;
    return $event;
}

# un-serializes, from a file
sub load {
    my ($class, $file) = @_;

    # use AS::Config to help with serialization
    my $cfg = ActiveState::Config->new();
    $cfg->parse_file( $file );
    my %hash = $cfg->as_hash();

    # bless the hash into an Event and return it to the caller
    my $event = \%hash;
    bless $event, $class;
    return $event;
}

1;

=head1 NAME

PureMessage::Monitor::Event - Basic event class

=head1 SYNOPSIS

  use PureMessage::Monitor::Event;
  ...
  $event = PureMessage::Monitor::Event->new(
                %data,
                );

  # query name of event
  $name = $event->name();

  # set/query event data parameters
  $event->param( 'key', 'value' );
  $value = $event->param( 'key' );

  # get event as a serialized string
  $str = $event->as_string();

  # parse event from a serialized string
  $event = PureMessage::Monitor::Event->parse( $str );

  # save event to disk
  $event->save( "event.txt" );

  # load event from disk
  $event = PureMessage::Monitor::Event->load( "event.txt" );

=head1 DESCRIPTION

C<PureMessage::Monitor::Event> is a basic event class, used for both
notification Events and Action Events.

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
