package PureMessage::Monitor::EventQueue::dir;

use strict;
use warnings;
use Carp;
use DirHandle;
use POSIX qw(strftime);
use Time::HiRes qw(gettimeofday);

# Creates a new directory-based event queue.
sub new {
    my ($class, $dir, %opts) = @_;

    # sanity check arguments
    if (!$dir) {
        croak "can't create queue directory with unspecified directory path";
    }

    # create a new object and bless it into our namespace.
    my $self = {
        %opts,
        'dir'   => $dir,
        };
    bless $self, $class;

    # auto-create our Queue directory if it doesn't exist yet
    if ($opts{'create'}) {
        $self->_create_qdir();
    }

    # sanity check that the queue directory exists
    if (!-d $dir) {
        croak "queue directory '$dir' missing";
    }

    # return the newly created object back to the caller.
    return $self;
}

# Adds a single item to the event queue, and returns its Queue ID.  Throws a
# fatal exception if we're unable to save the item to the queue.
sub add_item {
    my ($self, $event) = @_;

    # create a Queue ID for this event
    my $ts  = $event->param( 'time_stamp' );
    my $qid = _make_qid( $ts );

    # store the event out to disk
    my $path = $self->_qid_to_path( $qid );
    $event->save( $path );

    # return the Queue ID back to the caller.
    return $qid;
}

# Removes an item from the queue, by Queue ID
sub remove_item {
    my ($self, $qid) = @_;

    # remove the item from disk
    my $path = $self->_qid_to_path( $qid );
    unlink( $path );
}

# Returns a list of Queue IDs for items in the queue, in sorted order
sub items {
    my ($self) = @_;

    # open up our queue directory and get the list of files that it contains
    my $dirpath = $self->{'dir'};
    my $dirh    = DirHandle->new( $dirpath );
    my @items   = grep { -f "$dirpath/$_" } $dirh->read();

    # return the list of items to the caller, in sorted order
    return sort @items;
}

# Retrieves an item from the queue, by Queue ID
sub get_item {
    my ($self, $qid) = @_;

    # read the item in from disk.
    my $path  = $self->_qid_to_path( $qid );
    my $event = PureMessage::Monitor::Event->load( $path );

    # return the event back to the caller.
    return $event;
}

# Takes a Queue ID and returns the full path to the file that contains it.
sub _qid_to_path {
    my ($self, $qid) = @_;
    return $self->{'dir'} . "/$qid";
}

# Creates the Queue directory if it doesn't exist yet.
sub _create_qdir {
    my ($self) = @_;
    my $qdir = $self->{'dir'};
    if (!-d $qdir) {
        mkdir( $self->{'dir'}, 0755 ) or
            croak "can't create event queue directory $qdir; $!";
    }
}

# Creates a unique Queue ID (optionally based on a provided time stamp [in
# seconds since the epoch]), and returns the Queue ID back to the caller.
sub _make_qid {
    my ($timestamp) = @_;

    # get a hi-res time; we'll need the 'ms' resolution so that we can generate
    # unique Queue IDs
    my ($now, $now_ms) = gettimeofday();

    # default timestamp is "now" unless we were provided with one.
    if (!$timestamp) {
        $timestamp = $now;
    }

    # create a Queue ID in the form of "YMDHMS.now_ms.pid.rand"
    my $qid = join( '.',
                strftime('%Y%m%d%H%M%S', localtime($timestamp)),
                sprintf('%06d', $now_ms),
                $$,
                int rand(65536),
                );
    return $qid;
}

1;

=head1 NAME

PureMessage::Monitor::EventQueue::dir - Directory based event queue

=head1 SYNOPSIS

  use PureMessage::Monitor::EventQueue::dir;
  ...
  $queue = PureMessage::Monitor::EventQueue::dir->new(
                $queue_dir,
                );

  # adding items to the queue
  foreach my $event (@events) {
      $queue->add_item( $event );
  }

  # get list of Queue IDs for items in queue
  @queue_ids = $queue->items();

  # get an event in queue by its Queue ID
  $event = $queue->get_item( $queue_id );

  # remove an event from the queue, by its Queue ID
  $queue->remove_item( $queue_id );

=head1 DESCRIPTION

C<PureMessage::Monitor::EventQueue::dir> implements a simple directory based
queue for events.  As events are added to the queue they are given a unique
Queue ID (which is based on the time stamp of the event, if available), which
can then be used to retrieve/remove the event from the queue at a later time.

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
