package PureMessage::Monitor::Notifier::MessageLog;

use strict;
use warnings;
use Carp;
use Date::Parse qw(str2time);
use PureMessage::Monitor::Event;
use PureMessage::Monitor::LogReader;
use ActiveState::DateTime qw(gmt_time_stamp);

# fields that we extract out of the message_log and what we map them to in
# generated events.
my %message_log_fields = (
    # mlog  => mapped name in generated event
    'a'     => 'milter_status',
    'b'     => 'blocklist_result',
    'f'     => 'env_from',
    'fur'   => 'ext_relay_host',
    # comment 'h' out for now until we see how big the events are and how many
    # we want/need to generate and analyze
    #'h'     => 'spam_rule',
    'i'     => 'internal',
    'p'     => 'spam_probability',
    'q'     => 'queue_id',
    'r'     => 'relay_host',
    'size'  => 'message_size',
    't'     => 'env_to',
    'tm'    => 'processing_time',
    'v'     => 'virus_id',
    );

# Creates a new "message_log" notifier, and sets up the list of events that we
# notify others about.  Acceptable options:
#   event       - Hash-ref of events we're looking for in the message_log
#   logger      - Reference to logger object
sub new {
    my ($proto, $name, %opts) = @_;

    # sanity check parameters.
    if (!$name) {
        croak "can't create un-named message log notifier";
    }
    if (!$opts{'event'}) {
        croak "can't create message log notifier without event to notify on";
    }

    # create a new object and bless it into our namespace.
    my $class = ref($proto) || $proto;
    my $self = {
        'name'  => $name,
        'event' => $opts{'event'},      # ok to use hash-ref here; read only
        (defined $opts{'logger'} ? ('logger'=>$opts{'logger'}) : ()),
        };
    bless $self, $class;

    # return the newly created object back to the caller.
    return $self;
}

# returns the notifier name
sub name {
    return shift->{'name'};
}

# Runs the notifier; we'll scan through the message log for events and fire
# them off so that others can make use of them.
sub run {
    my ($self, $queue) = @_;
    my $logger = $self->{'logger'};
    $logger->log( 'DEBUG', "running $self->{name} notifier" ) if ($logger);

    # create a log reader for the "message_log"
    $logger->log( 'DEBUG', "opening message log for reading" ) if ($logger);
    my $reader = PureMessage::Monitor::LogReader->new( 'type' => 'message' );
    $reader->open();

    # read through the message log, look for events, and notify others about
    # them by adding them to the queue.
    while (my $data = $reader->fetchrow_hashref()) {
        # get timestamp of the current line
        my ($time_stamp, $gmt_offset) = gmt_time_stamp($data->{'time_stamp'});
        $time_stamp .= $gmt_offset;

        # check for each of the events that we notify others about
        while (my ($name, $info) = each %{$self->{'event'}}) {
            # filter on all fields/regexes specified
            my $match = 1;
            while (my ($f_field, $f_regex) = each %{$info}) {
                if (not ($data->{$f_field} && ($data->{$f_field} =~ /$f_regex/))) {
                    $match = 0;
                }
            }

            # if we've found a match then fire an event off into the queue
            if ($match) {
                my %event_data = (
                    # meta-data
                    'name'          => $name,
                    'time_stamp'    => str2time( $time_stamp ),
                    );
                foreach my $field (keys %message_log_fields) {
                    if (defined $data->{$field}) {
                        my $mapped = $message_log_fields{$field};
                        $event_data{$mapped} = $data->{$field};
                    }
                }
                $logger->log( 'NOTICE', "event '$name' match found, firing event" ) if ($logger);
                my $event = PureMessage::Monitor::Event->new( %event_data );
                $queue->add_item( $event );
            }
        }
    }

    # all done; save state as to how far we've read into the "message_log"
    $logger->log( 'DEBUG', "saving message log state so we can resume later" ) if ($logger);
    $reader->save_resume_info();
}

1;

=head1 NAME

PureMessage::Monitor::Notifier::MessageLog - monitors message_log for events

=head1 SYNOPSIS

  <notifier message_log>
      use = PureMessage::Monitor::Notifier::MessageLog
      <event spam>
        p = 0\.[56789]|1\.
      </event>
      <event internal_mail>
        i=.
      </event>
      <event every_message>
        q=.
      <event>
  </notifier>

=cut
