package PureMessage::Monitor::Reactor::Email;

use strict;
use warnings;
use Carp;
use PureMessage::Config qw(%config expand_tmpl_filename);
use PureMessage::Template qw(expand_template);
use PureMessage::Queue qw(queue_mail);

use base qw( PureMessage::Monitor::EventHandler );

my %DEFAULTS = (
    'from'      => $config{'admin'},
    'subject'   => 'e-mail alert',
    'template'  => 'monitor/reactor-email.tmpl',
    );

# Creates a new "e-mail" reactor.  Acceptable options:
#   to          - e-mail address msg is sent to
#   from        - e-mail address msg is from (default PureMessage Admin)
#   subject     - subject for e-mail message
#   template    - file containing message template
sub new {
    my $proto = shift;

    # create a new object
    my $self = $proto->SUPER::new( @_ );

    # sanity check parameters
    if (!$self->{'to'}) {
        my $name = $self->name();
        croak "can't create reactor '$name' without specifying who to send e-mails to";
    }

    # use defaults where necessary
    foreach my $key (keys %DEFAULTS) {
        if (!defined $self->{$key}) {
            $self->{$key} = $DEFAULTS{$key};
        }
    }

    # return the newly created object back to the caller.
    return $self;
}

# reacts to the given event, by sending an e-mail message
sub react_on_event {
    my ($self, $event) = @_;

    # slurp in the body of the template that we're using
    my $template = expand_tmpl_filename( $self->{'template'} );
    my $message;
    if (open( my $FIN, $template)) {
        $message = join( '', <$FIN> );
        close( $FIN );
    }
    else {
        croak "unable to access e-mail template '$template'; $!";
    }

    # expand out the template
    $message = expand_template( $message,
                    'vars' => {
                        $event->as_hash(),
                        'subject'   => $self->{'subject'},
                        'from'      => $self->{'from'},
                    }, );

    # queue up the message
    queue_mail(
        'from'      => $self->{'from'},
        'to'        => $self->{'to'},
        'message'   => $message,
        );
    my $event_name = $event->name();
    $self->log( 'INFO', "e-mail notification about $event_name sent" );
}

1;

=head1 NAME

PureMessage::Monitor::Reactor::Email - React by sending e-mail messages

=head1 SYNOPSIS

    <reactor send_alert_email>
        use = PureMessage::Monitor::Reactor::Email
        event = alert
        from = root@example.com
        to = user@example.com
        subject = ARGH!
        template = alert-email.tmpl
    </reactor>

    <reactor send_alert_broadcast>
        use = PureMessage::Monitor::Reactor::Email
        event = alert
        from = root@example.com
        <to>
            user@example.com
            root@example.com
            postmaster@example.com
        </to>
        subject = ARGH!
        template = alert-broadcast.tmpl
    </reactor>

=cut
