package PureMessage::Monitor::Reactor::ListAdd;

use strict;
use warnings;
use Carp;
use PureMessage::List;

use base qw( PureMessage::Monitor::EventHandler );

# Creates a new "add to list" reactor.  Acceptable options:
#   list        - name of list to add item to (REQUIRED)
sub new {
    my $proto = shift;

    # create a new object
    my $self = $proto->SUPER::new( @_ );

    # sanity check parameters
    if (!$self->{'list'}) {
        my $name = $self->name();
        croak "can't create reactor '$name' without specifying list to add items to";
    }

    # return the newly created object back to the caller
    return $self;
}

# reacts to the given action event, adding the item to a list
sub react_on_event {
    my ($self, $event) = @_;

    # get the value out of the event
    my $value = $event->param( 'value' );

    # get a handle to the listdata for the list that we're adding the item to
    my $listname = $self->{'list'};
    my $list = eval { PureMessage::List->new($listname) };
    if ($@) {
        croak "can't access list '$listname'; $@";
    }

    my $listdata = $list->data();
    if ($listdata->is_readonly()) {
        croak "can't write to list '$listname'; is read-only";
    }

    # if the item isn't already in the list, add it to the list
    my $found = 0;
    $listdata->enumerate_items( sub { $found=1 if ($value eq shift) } );
    if (!$found) {
        $self->log( 'INFO', "adding '$value' to '$listname'" );
        $listdata->add_items( $value );
        $listdata->commit();
    }
}

1;

=head1 NAME

PureMessage::Monitor::Reactor::ListAdd - React by adding item to list

=head1 SYNOPSIS

    <reactor add_to_blacklist>
        use = PureMessage::Monitor::Reactor::ListAdd
        event = too_much_spam_from_host
        list = blacklisted-hosts
    </reactor>

=cut
