package PureMessage::Monitor::Reactor::ListAddTemporary;

use strict;
use warnings;
use Carp;
use PureMessage::Monitor::TemporaryList;

use base qw( PureMessage::Monitor::EventHandler );

my %DEFAULTS = (
    'expiry'    => 3600,
    );

# Creates a new "add to list temporarily" reactor.
#
# Acceptable options:
#   list        - name of list to add items to (REQUIRED)
#   expiry      - expiry period for temporary data (default 1h)
sub new {
    my $proto = shift;

    # create a new object
    my $self = $proto->SUPER::new( @_ );

    # sanity check parameters
    if (!$self->{'list'}) {
        my $name = $self->name();
        croak "can't create reactor '$name' without specifying list to temporarily add items to";
    }

    # use defaults where necessary
    foreach my $key (keys %DEFAULTS) {
        if (!defined $self->{$key}) {
            $self->{$key} = $DEFAULTS{$key};
        }
    }

    # create a temporary list to add items to
    $self->{'temporary'} = PureMessage::Monitor::TemporaryList->new(
                            $self->{'list'},
                            'expiry'    => $self->{'expiry'},
                            );

    # return the newly created object back to the caller
    return $self;
}

# reacts to the given action event, temporarily adding the item to a list
sub react_on_event {
    my ($self, $event) = @_;

    # get the value out of the event
    my $value = $event->param( 'value' );

    # add the item to our list, temporarily
    $self->log( 'DEBUG', "adding '$value' temporarily to $self->{list} list" );
    $self->{'temporary'}->add_items( $value );
}

1;

=head1 NAME

PureMessage::Monitor::Reactor::ListAddTemporary - React by adding item to list temporarily

=head1 SYNOPSIS

    <reactor temp_add_to_blacklist>
        use = PureMessage::Monitor::Reactor::ListAddTemporary
        event = too_much_spam_from_host
        list = blacklisted-hosts
        expiry = 1h
    </reactor>

=cut
