package PureMessage::Monitor::Reactor::Log;

use strict;
use warnings;
use Carp;
use PureMessage::Template qw(expand_template);
use PureMessage::Logger;

use base qw( PureMessage::Monitor::EventHandler );

my %DEFAULTS = (
    'logfile'   => 'reactor_log',
    'format'    => q{'%%NAME%%' occurred at %%TIME_STAMP_STR%%},
    );

# Creates a new "logger" reactor.  Acceptable options:
#   logfile     - name of logfile to log to
#   format      - log format (can include template variables)
sub new {
    my $proto = shift;

    # create a new object
    my $self = $proto->SUPER::new( @_ );

    # use defaults where necessary
    foreach my $key (keys %DEFAULTS) {
        if (!defined $self->{$key}) {
            $self->{$key} = $DEFAULTS{$key};
        }
    }

    # return the newly created object back to the caller.
    return $self;
}

# reacts to the given event, by writing a log entry stating that it occurred
sub react_on_event {
    my ($self, $event) = @_;

    # template expand the format we're writing out to the log
    my $expanded = expand_template( $self->{'format'},
                        'vars'  => {
                            $event->as_hash(),
                        }, );
                                    
    # open up our logfile and write our log entry out to it
    my $logger = PureMessage::Logger->new(
                    'log_to'    => $self->{'logfile'},
                    );
    $logger->log( 'NOTICE', $expanded );
}

1;

=head1 NAME

PureMessage::Monitor::Reactor::Log - React by writing an entry to a logfile

=head1 SYNOPSIS

    <reactor audit_log>
        use = PureMessage::Monitor::Reactor::Log
        event = all
        logfile = monitor_audit_log
        format = %%NAME%% occurred at %%TIME_STAMP_STR%%
    </reactor>

=cut
