package PureMessage::Monitor::TemporaryList;

use strict;
use warnings;
use Carp;
use base qw( PureMessage::Monitor::TimedCounter );
use PureMessage::Version qw($PREFIX);
use PureMessage::List;

# Returns the names of all of the known temporary lists.
sub Available {
    my $path = "$PREFIX/var/datastore/temporary_list";
    my @paths;
    foreach my $item (<$path/*>) {
        if (-d $item) {
            $item =~ s{^$path/}{};
            push( @paths, $item );
        }
    }
    return @paths;
}

# Creates a new temporary list, returning a reference to the newly created
# object back to the caller.
sub new {
    my ($proto, $dir, %args) = @_;
    $args{'store'} = 'temporary_list';
    my $self = $proto->SUPER::new( $dir, %args );
    return $self;
}

# Refreshes the PM::List object for this TemporaryList, to the current state of
# the data in the TemporaryList.  The PM::List is modified by only adding or
# removing new/expired items from the list; it -IS- safe to do this to a list
# that has user-entered data in it.
sub refresh {
    my $self = shift;

    # get a handle to the PM::List we're updating
    my $list = PureMessage::List->new( $self->name() );
    my $data = $list->data();
    if ($data->is_readonly()) {
        croak "can't refresh contents of read-only list";
    }

    # purge stale data, and remove purged items from the PM::List
    $self->purge(
        'on_item_purge' => sub {
            my $item = shift;
            $data->remove_items( $item );
        } );

    # get the items in the PM::List and in the store, and figure out which ones
    # need to be added to the PM::List.
    my %listitems;
    $data->enumerate_items( sub { $listitems{shift}++ } );

    my @items = $self->items();
    foreach my $item (@items) {
        if (!exists $listitems{$item}) {
            $data->add_items( $item );
        }
    }

    # commit our changes to the list
    $data->commit();
}

# Adds items to the list.  Existing list items are refreshed and their expiry
# period is pushed further into the future, items are -not- duplicated in the
# list.
sub add_items {
    my $self = shift;
    foreach my $item (@_) {
        # CHEAT: actually, use "increment()" here; we're just providing a more
        # list-like function name than "increment()" so its more familiar.
        $self->increment( $item );
    }
}

1;

=head1 NAME

PureMessage::Monitor::TemporaryList - Add items to list, temporarily

=head1 SYNOPSIS

  use PureMessage::Monitor::TemporaryList;
  ...
  # manipulate "blacklisted-hosts", where items expire in 1hr
  $list = PureMessage::Monitor::TemporaryList(
            'blacklisted-hosts',
            'expiry' => 3600,
            );

  # add items to the list, temporarily
  $list->add_items( 'example.com', 'foobar.com' );

  # refresh the underlying list data
  $list->refresh();

=head1 DESCRIPTION

C<PureMessage::Monitor::TemporaryList> provides a mechanism for adding items to
a PureMessage list, temporarily.  Create a
C<PureMessage::Monitor::TemporaryList> object telling it how long newly added
items are allowed to remain in the list before they expire, and add items to
the list.  Refresh the temporary list, and changes will be pushed through to
the underlying PureMessage List (both adds and removes).

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
