package PureMessage::Monitor::TimedCounter;

use strict;
use warnings;
use Carp;
use PureMessage::DataStore;

# Default values
my $DEF_EXPIRY = 300;           # 5m default expiry
my $DEF_PERIOD = 60;            # 1m default period duration
my $DEF_STORE  = 'timed_counter_store';

# Min/max values
# - Enforced to help ensure a predictable behaviour for the object; its not
#   because of any inherent limitation in the module or how we store things.
my $MIN_EXPIRY = 60;            # minimum 1m expiry
my $MAX_EXPIRY = 86400;         # maximum 1d expiry
my $MIN_PERIOD = 60;            # minimum 1m period duration
my $MAX_PERIOD = 86400;         # maximum 1d period duration

# Creates a new timed counter.  Accepted args:
#   expiry      - expiry time for items, in seconds (default 300s)
#   period      - how long a period is, in seconds (default 60s)
sub new {
    my ($proto, $dir, %args) = @_;

    # sanity check incoming arguments
    if (!$dir) {
        croak "can't create un-named timed counter";
    }
    if ($args{'expiry'}) {
        if ($args{'expiry'} < $MIN_EXPIRY) {
            croak "can't create TimedCounter with expiry less than $MIN_EXPIRY seconds";
        }
        if ($args{'expiry'} > $MAX_EXPIRY) {
            croak "can't create TimedCounter with expiry more than $MAX_EXPIRY seconds";
        }
    }
    if ($args{'period'}) {
        if ($args{'period'} < $MIN_PERIOD) {
            croak "can't create TimedCounter with period smaller than $MIN_PERIOD seconds";
        }
        if ($args{'period'} > $MAX_PERIOD) {
            croak "can't create TimedCounter with period larger than $MAX_PERIOD seconds";
        }
    }
    my $store = $DEF_STORE;
    if ($args{'store'}) {
        $store = $args{'store'};
    }

    # create a new object and bless it into our namespace
    my $class = ref($proto) || $proto;
    my $self = {
        'dir'      => $dir,
        'store'    => PureMessage::DataStore->new( 'store' => "dir:'$store/$dir'" ),
        'expiry'   => $args{'expiry'} ? $args{'expiry'} : $DEF_EXPIRY,
        'period'   => $args{'period'} ? $args{'period'} : $DEF_PERIOD,
        };
    bless $self, $class;

    # return newly created object back to the caller
    return $self;
}

# Returns the name of the timed counter
sub name {
    return shift->{'dir'};
}

# Returns the expiry period for the timed counter.
sub expiry {
    return shift->{'expiry'};
}

# Returns the period duration for the timed counter.
sub period {
    return shift->{'period'};
}

# Returns the count for the given item, taking expiry of old/stale values into
# account.
sub count {
    my ($self, $item) = @_;
    my $count = 0;

    # how old is too old?
    my $too_old = time();

    # get the on-disk data store for the item.
    my $data = $self->{'store'}->get( $item );

    # count up the data for this item
    foreach my $period ($data->contents()) {
        # skip meta info
        next if $period =~ /^_/;
        # only count data if its not too old
        if ($period >= $too_old) {
            my $period_count = $data->retrieve( $period );
            $count += $period_count;
        }
    }

    # return the total count back to the caller
    return $count;
}

# Increments the counter for the specified item, with optional creation time.
# If no creation time is provided, we default to "now".
sub increment {
    my ($self, $item, %opts) = @_;

    # extract options and set defaults where necessary
    my $now = $opts{'now'} || time();

    # determine when this item expires
    my $expires = $now + $self->{'expiry'};

    # determine the period that this item should be created in
    my $period = $self->_time_to_period( $expires );

    # increment the count for the given period for this item
    my $data  = $self->{'store'}->get( $item );
    my $count = $data->retrieve( $period );
    $count ++;
    $data->store( $period, $count );
}

sub flag {
    my $self  = shift;
    my $item  = shift;
    my $name  = '_' . shift;

    my $data   = $self->{'store'}->get( $item );
    my $value  = $data->retrieve( $name );

    if (@_) {
        my $req_status = shift;
        if (!$value && $req_status) {
            $value = time();
            $data->store( $name, $value );
        }
        elsif ($value && !$req_status) {
            $data->delete( $name );            
        }
    }
    return ( $value );
}

# Returns the list of data items currently being counted.
sub items {
    my $self = shift;

    # how old is too old?
    my $too_old = time();

    # build up the list of items being counted
    my @items;
    foreach my $item ($self->{'store'}->keys()) {
        my $data = $self->{'store'}->get( $item );
        # if data item has any current periods, add them to our list
        if (grep { $_ >= $too_old } $data->contents()) {
            push( @items, $item );
        }
    }

    # return the list of items back to the caller
    return @items;
}

# Purges stale data from the counter.
#
# You can use the 'on_data_purge' and 'on_item_purge' callbacks to keep track
# of what got purged:
#
#   on_data_purge
#       - called when a count is about to be purged because its stale
#       - passed parameters of "item", "count", "period count was in"
#
#   on_item_purge
#       - called when an item is being purged because it has no counts
#         associated with it
#       - passed parameters of "item"
sub purge {
    my ($self, %opts) = @_;

    # get callbacks from provided options
    my $cb_data_purge = $opts{'on_data_purge'} || sub { };
    my $cb_item_purge = $opts{'on_item_purge'} || sub { };

    # how old is too old?
    my $too_old = time();

    # purge stale data for all items in data store
    foreach my $item ($self->{'store'}->keys()) {
        my $data = $self->{'store'}->get( $item );
        # delete any stale periods for this data item
        foreach my $period ($data->contents()) {
            # skip meta info
            next if $period =~ /^_/;
            if ($period < $too_old) {
                if ($cb_data_purge) {
                    my $count = $data->retrieve( $period );
                    $cb_data_purge->( $item, $count, $period );
                }
                $data->delete( $period );
            }
        }
        # delete the data item if its got no data
        my @contents = $data->contents();
        if (!@contents) {
            if ($cb_item_purge) {
                $cb_item_purge->( $item );
            }
            $self->{'store'}->delete( $item );
        }
    }
}

# INTERNAL METHOD: returns the period for the given time (in seconds since the
# epoch).  "Period" is considered to be granular "to the minute".
sub _time_to_period {
    my ($self, $time_t) = @_;
    my $period = $time_t - ($time_t % $self->{'period'});
    return $period;
}

1;

=head1 NAME

PureMessage::Monitor::TimedCounter - Counter with timed data expiration

=head1 SYNOPSIS

  use PureMessage::Monitor::TimedCounter;
  ...
  $counter = PureMessage::Monitor::TimedCounter(
                'unique_counter_name',
                'expiry' => 300,
                'period' => 60,
                );

  # increment counter for item with default creation time (now)
  $counter->increment( 'item' );

  # increment counter for item with explicit creation time
  $counter->increment( 'item', 'now' => $creation_time );

  # current count for item
  $count = $counter->count( 'item' );

  # purge stale/old data items
  $counter->purge();

  # purge, with callbacks
  $counter->purge(
    'on_data_purge' => sub { push @purged_data, [@_] },
    'on_item_purge' => sub { push @purged_item, shift },
    );

=head1 DESCRIPTION

C<PureMessage::Monitor::TimedCounter> provides a counter mechanism with timed
data expiration.  Counts can be incremented for a given item and then expired
based on their age.

When asked for the total count for a particular item,
C<PureMessage::Monitor::TimedCounter> returns the count of I<current> items,
already taking into consideration that there may be items that are stale/old
and that aren't relevant.  Periodically, though, you will want to C<purge()>
stale items from the timed counter, to ensure proper cleanup.

=head1 COPYRIGHT

Copyright (C) 2000-2006 Sophos Group.  All rights reserved.  Sophos and
PureMessage are trademarks of Sophos Plc and Sophos Group.

=cut
